// AxsunOCTControl_LW_C.h
// Copyright 2022 Excelitas Technologies

/** \file AxsunOCTControl_LW_C.h
\brief This header file defines the usage of the %AxsunOCTControl_LW "lightweight" library in a C or C++ application.
 
Refer to the Main Page for \link cusagesec C API Usage Guidelines \endlink.

DETAILED RELEASE NOTES

Version 1.0.0
- initial release

Version 1.0.1
- improved Linux OS compatbility
- compatible with libusb v1.0.24.11650 when using libusb-win32/libusb0.sys (v1.2.6) driver on Windows OS
- improved type safety when compiled as __cplusplus (uses "enum class")

Version 1.1.0
- added function axGetElapsedTimers() to query elapsed timer values from laser engine

Version 1.2.0
- axGetLaserEmission() and axSetLaserEmission() have independent query/control of laser emission versus sweep-only mode (e.g. for sweep trigger generation without laser emission)

Version 1.3.0
- moved AxEdgeSource definition from AxsunOCTControl_LW_C.h to AxsunCommonEnums.h
- support for Azymth laser engine
- support for synchronous internal Image_sync generation on FPGA version 6.71.0.42 or newer; see description for axSetImageSyncSource()
- __cdecl calling convention added explicitly to function declarations for Windows targets
- added function axGetLaserErrorState() to query potential error states reported by laser engine firmware

Version 1.3.1
- removed preprocessor logic corresponding to LabVIEW event integration and updated axRegisterLabVIEWCallback() parameter to uint32_t*

Version 1.4.0
- added function axSetFPGAWindowFunction() to upload real & imaginary components of the apodization window function to the DAQ FPGA. This convenience function avoids the need for a user to call axSetFPGADataArray() twice.

Version 1.5.0
- added functions axGetLWBuildDateTime() and axGetLWBuildCfg() to query library details and axGetLibusbVersion() to query loaded libusb dependency version
- added callback registration functions axRegCallbackConnectOnly() and axRegCallbackDisconnectOnly() to separate the behavior of the callback registered via axRegisterConnectCallback() which is invoked for both connection and disconnection events
- axSetImageSyncSource() supports selection of connector interface on exposed front edge of 900 MS/s DAQ board when installed in PCIe configuration

Version 1.6.0
- added function axGetDeviceInfo() to query device type, interface, FW & FPGA versions, and serial number in a single function call
- added functions axSetLaserPowerUp(), axGetLaserPowerUp(), axSetPointerPowerUp(), axGetPointerPowerUp() and enum AxPowerUpBehavior to control/query default behavior for the swept OCT laser and auxiliary pointer/aiming laser when the engine powers-up
- added header-only helper functions for enum -> string conversion in C++
- enabled lazy initialization of Control_LW session without explicit call to axOpenAxsunOCTControl(); this function is now optional.
 */

#ifndef AXSUNOCTCONTROLLW_C_H
#define AXSUNOCTCONTROLLW_C_H

#include "AxsunCommonEnums.h"

#ifdef __cplusplus
#include <cstdint>
extern "C" {
#else
#include <stdint.h>
#endif // __cplusplus

/*! \brief Connection types (interfaces) supported by AxsunOCTControl_LW. */
typedef enum
#ifdef __cplusplus
class
#endif // __cplusplus
AxConnectionType {
	NONE = 0,
	USB = 1,					/*!< USB 2.0 interface. */
	RS232_PASSTHROUGH = 2,		/*!< via DAQ's Ethernet interface. */
	RS232 = 3,					/*!< RS-232 serial interface. */
	ETHERNET = 4				/*!< TCP/IP Ethernet interface. */
} AxConnectionType;

/*! \brief Axsun device types supported by AxsunOCTControl_LW. */
typedef enum 
#ifdef __cplusplus
class
#endif // __cplusplus
AxDevType {
	UNDEFINED = 0,
	LASER = 40,			/*!< Axsun Laser Engine Device. */
	CLDAQ = 41,			/*!< Axsun Camera Link DAQ Device. */
	EDAQ = 42,			/*!< Axsun Ethernet/PCIe DAQ Device. */
	AZMYTH_LASER = 43	/*!< Axsun Azmyth Laser Engine Device. */
} AxDevType;

/*! \brief Laser TEC states. */
typedef enum 
#ifdef __cplusplus
class
#endif // __cplusplus
AxTECState {
	TEC_UNINITIALIZED = 0,			/*!< TEC has not been initialized. */
	WARMING_UP = 1,					/*!< TEC is stabilizing. */
	WAITING_IN_RANGE = 2,			/*!< TEC is stabilizing. */
	READY = 3,						/*!< TEC is ready. */
	NOT_INSTALLED = 5,				/*!< TEC is not installed. */
	ERROR_NEVER_GOT_TO_READY = 16,	/*!< TEC error: never got to ready state. */
	ERROR_WENT_OUT_OF_RANGE = 17,	/*!< TEC error: temperature went out of range. */
	TEC_STATE_UNAVAILABLE = 99
} AxTECState;

/*! \brief Laser engine default/power-up behaviors. */
typedef enum
#ifdef __cplusplus
class
#endif // __cplusplus
AxPowerUpBehavior {
	BEHAVIOR_UNAVAILABLE = 0,
	OFF_AT_POWER_UP = 1,			/*!< Laser emission disabled at power-up. */
	ON_AT_POWER_UP = 2,				/*!< Laser emission enabled at power-up. */
	USE_DIGITAL_INPUT = 3,			/*!< Swept laser state depends on digital input (Pin 1 on engine's Auxiliary & Interlocks connector). */
	LINK_TO_SWEPT_LASER = 4,		/*!< Aiming laser emission state tracks that of swept laser */
	BEHAVIOR_RFU = 5
} AxPowerUpBehavior;

/** \brief Structure for device information available in connect or disconnect callback functions. */
struct AxDeviceInfo {
	AxDevType device_type;				/*!< device type */
	AxConnectionType connection_type;	/*!< connection type */
	char serial_number[40];				/*!< device serial number string (nul-terminated) */
	char firmware_version[16];			/*!< device firmware version string (nul-terminated) */
	char FPGA_version[16];				/*!< device FPGA version string, if applicable (nul-terminated) */
};
typedef struct AxDeviceInfo AxDeviceInfo;

/*! \brief Type defintion for a user-provided function to be invoked upon connection or disconnection of a device, when registered using axRegisterConnectCallback(). */
typedef void(__cdecl *AxConnectCallbackFunctionC_t)(void*);

/*! \brief Type defintion for a user-provided function to be invoked upon device connection if registered using axRegCallbackConnectOnly() or upon device disconnection if registered using axRegCallbackDisconnectOnly(). */
typedef void(__cdecl* AxCallbackConnectOrDisconnect_t)(AxDeviceInfo, int32_t, void*);

/** @name Generic Library Functions
 *  These functions are not directly related to a connected device, but are used for opening & closing the Control_LW session, opening & closing communication interfaces, and other session management.
 */
///@{

/**
 \brief Opens the AxsunOCTControl_LW session.
 \param[in] open_all_interfaces If TRUE the USB and Ethernet interfaces will be started automatically. If FALSE no interfaces will be started.
 \return NO_AxERROR on success or other AxErr error code on failure.
 \details See \link cusagesec C API Usage Guidelines \endlink on the Main Page for more information.
 \details If interfaces are not opened at the time of this function call, they can be opened later using axNetworkInterfaceOpen(), axUSBInterfaceOpen(), or axSerialInterfaceOpen(). This allows the registration of connection/disconnection callback function(s) _prior to_ opening interfaces and gives the user individual control over which interface(s) to use.
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axOpenAxsunOCTControl(uint32_t open_all_interfaces);

/**
 \brief Closes an AxsunOCTControl_LW session previously opened with axOpenAxsunOCTControl().
 \return NO_AxERROR on success or other AxErr error code on failure. */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axCloseAxsunOCTControl();

/**
 \brief Counts the number of devices successfully connected and enumerated by AxsunOCTControl_LW.
 \return The number of connected devices.
 \details Count includes Laser engine and DAQ boards. A device connected on two interfaces concurrently will be counted twice and thus is not a recommended configuration.
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
uint32_t __cdecl axCountConnectedDevices();

/**
 \brief Gets a string which explains an error code in a more verbose fashion.
 \param[in] errornum An error code number returned from other "ax" functions in this library.
 \param[out] error_string A char array pre-allocated with at least 512 bytes into which the error explanation will be copied as a nul-terminated string.
 \return n/a
 \details axGetErrorExplained() can be called at any time. It is unsafe to pass a error_string output buffer allocated with fewer than 512 bytes.
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
void __cdecl axGetErrorExplained(AxErr errornum, char* error_string);

/**
 \brief Opens, resets, or closes the USB interface.
 \param[in] interface_status =1 opens the interface or resets an existing open interface, =0 closes the interface.
 \return NO_AxERROR on success or other AxErr error code on failure.
 \details When the interface is open, devices present on this interface will be detected, enumerated, and trigger the execution of a callback function (if previously registered). Use this function if the interface was not opened automatically during the initial axOpenAxsunOCTControl() call.
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axUSBInterfaceOpen(uint32_t interface_status);

/**
 \brief Opens, resets, or closes the Ethernet network interface.
 \param[in] interface_status =1 opens the interface or resets an existing open interface, =0 closes the interface.
 \return NO_AxERROR on success or other AxErr error code on failure.
 \details When the interface is open, devices present on this interface will be detected, enumerated, and trigger the execution of a callback function (if previously registered). Use this function if the interface was not opened automatically during the initial axOpenAxsunOCTControl() call.
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axNetworkInterfaceOpen(uint32_t interface_status);

/**
 \brief Opens, resets, or closes a RS-232 serial interface on a given port.
 \param[in] interface_status =1 opens an interface or resets an existing open interface, =0 closes an interface if a `port` is provided, or closes all interfaces if `port` is "" (empty string).
 \param[in] port Nul-terminated char array describing the name of the port (e.g. typically "COM_" on Windows or "/dev/tty.___" on Linux/macOS; UTF8-encoded)
 \return NO_AxERROR on success or other AxErr error code on failure.
 \details When the interface is open, a device present on this interface will be detected, enumerated, and trigger the execution of a callback function (if previously registered).
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axSerialInterfaceOpen(uint32_t interface_status, const char* port);

/**
 \brief Gets the AxsunOCTControl_LW library version.
 \param[out] major Version number major element.
 \param[out] minor Version number minor element.
 \param[out] patch Version number patch element.
 \param[out] build Version number build element.
 \return NO_AxERROR on success.
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axLibraryVersion(uint32_t* major, uint32_t* minor, uint32_t* patch, uint32_t* build);

/**
 \brief Get a string describing the time and date that the AxsunOCTControl_LW library was built.
 \param[out] datetime_str A pointer to a pre-allocated output buffer of characters with size at least 100 bytes into which a nul-terminated date & time string will be copied.
 \return The number of characters (bytes) written into the datetime_str buffer, not including the trailing nul.
 \details It is unsafe to pass a datetime_str output buffer allocated with fewer than 100 bytes.
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
int32_t __cdecl axGetLWBuildDateTime(char* datetime_str);

/**
\brief Get the build configuration of the AxsunOCTControl_LW library.
\return = a bitfield with information about the build configuration options used to compile the loaded library.
\ Bit 0: = 0 for 'debug' configuration or = 1 for 'release' configuration
\ Bit 1: (for internal use only)
\ Bit 2: = 0 for non-verbose mode or = 1 for verbose logging mode
*/
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
int32_t __cdecl axGetLWBuildCfg();

/**
 \brief Gets the version of the libusb library loaded with axUSBInterfaceOpen().
 \param[out] major Version number major element.
 \param[out] minor Version number minor element.
 \param[out] patch Version number patch element.
 \param[out] build Version number build element.
 \return NO_AxERROR on success or HOSTSIDE_USB_ERROR if the USB interface has not yet been opened.
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axGetLibusbVersion(uint32_t* major, uint32_t* minor, uint32_t* patch, uint32_t* build);

/**
 \brief Registers a callback function to be executed following a device connection or disconnection event.
 \param[in] callback_function A user-supplied function to be called.
 \param[in] userData An arbitrary structure to be made available inside the callback function (or NULL if unused).
 \return NO_AxERROR on success or other AxErr error code on failure.
 \details The supplied callback function will be executed by a background thread when a new device is detected or when an existing device is disconnected. Within the callback function, access is provided to arbitrary user data passed as a void* parameter to this function.  It is recommended that new designs use axRegCallbackConnectOnly() and axRegCallbackDisconnectOnly() if discrimination between device connection and disconnection events is desired.
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axRegisterConnectCallback(AxConnectCallbackFunctionC_t callback_function, void* userData);

/**
 \brief Registers a callback function to be executed following a device connection event.
 \param[in] callback_function A user-supplied function to be called.
 \param[in] userData An arbitrary structure to be made available inside the callback function (or NULL if unused).
 \return NO_AxERROR on success or other AxErr error code on failure.
 \details The supplied callback function will be executed by a background thread when a new device is detected. Within the callback function, access is provided to arbitrary user data passed as a void* parameter to this function.
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axRegCallbackConnectOnly(AxCallbackConnectOrDisconnect_t callback_function, void* userData);

/**
 \brief Registers a callback function to be executed following a device disconnection event.
 \param[in] callback_function A user-supplied function to be called.
 \param[in] userData An arbitrary structure to be made available inside the callback function (or NULL if unused).
 \return NO_AxERROR on success or other AxErr error code on failure.
 \details The supplied callback function will be executed by a background thread when an existing device is disconnected. Within the callback function, access is provided to arbitrary user data passed as a void* parameter to this function.
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axRegCallbackDisconnectOnly(AxCallbackConnectOrDisconnect_t callback_function, void* userData);

/**
 \brief Registers a LabVIEW user event to be executed following a device connection or disconnection event.
 \param[in] refnumptr A refnum pointer of a LabVIEW user event.
 \return NO_AxERROR on success or other AxErr error code on failure.
 \details LabVIEW code can be called directly from this library when a new device is detected or when an existing device is disconnected, similar to a C-based callback function registered using axRegisterConnectCallback(). See the %AxsunOCTControl_LW LabVIEW example project and _extcode.h_ header file (provided by National Instruments) for more details.
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axRegisterLabVIEWCallback(uint32_t* refnumptr);


///@}


/** @name Generic Device Functions
 *  These functions access connected devices (either laser engine or DAQ).
 */
 ///@{



/**
 \brief Gets the device firmware version.
 \param[out] major Version number major element.
 \param[out] minor Version number minor element.
 \param[out] patch Version number patch element.
 \param[in] which_device The numeric index of the desired device.
 \return NO_AxERROR on success or other AxErr error code on failure.
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axFirmwareVersion(uint32_t* major, uint32_t* minor, uint32_t* patch, uint32_t which_device);

/**
 \brief Gets the device FPGA version.
 \param[out] major Version number major element.
 \param[out] minor Version number minor element.
 \param[out] patch Version number patch element.
 \param[out] build Version number build element.
 \param[in] which_device The numeric index of the desired device.
 \return NO_AxERROR on success or other AxErr error code on failure.
 \details Devices which do not have an FPGA will return all version elements = 0.
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axFPGAVersion(uint32_t* major, uint32_t* minor, uint32_t* patch, uint32_t* build, uint32_t which_device);

/**
 \brief Gets the device type (e.g. Laser, EDAQ).
 \param[out] device_type The type of device.
 \param[in] which_device The numeric index of the desired device.
 \return NO_AxERROR on success or other AxErr error code on failure.
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axDeviceType(AxDevType* device_type, uint32_t which_device);

/**
 \brief Gets the device serial number string.
 \param[out] serial_number A char array pre-allocated with at least 40 bytes into which the device serial number will be copied as a nul-terminated string.
 \param[in] which_device The numeric index of the desired device.
 \return NO_AxERROR on success or other AxErr error code on failure.
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axSerialNumber(char* serial_number, uint32_t which_device);

/**
 \brief Gets the device connection interface (e.g. USB, Ethernet, RS-232).
 \param[out] connection_type The interface on which the device is connected.
 \param[in] which_device The numeric index of the desired device.
 \return NO_AxERROR on success or other AxErr error code on failure.
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axConnectionType(AxConnectionType* connection_type, uint32_t which_device);

/**
 \brief Gets info about a connected device.
 \param[out] device_info Information about the desired device.
 \param[in] which_device The numeric index of the desired device.
 \return NO_AxERROR on success or other AxErr error code on failure.
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axGetDeviceInfo(AxDeviceInfo* device_info, uint32_t which_device);

///@}



/** @name Laser Device Functions
 *  These functions access laser engine devices only.
 */
 ///@{

/**
 \brief Enables or disables swept laser emission.
 \param[in] emission_state =0 disables both laser emission and sweep, =1 enables both laser emission and sweep, =2 enables sweep but disables laser emission.
 \param[in] which_laser The numeric index of the desired Laser.
 \return NO_AxERROR on success or other AxErr error code on failure.
 */
/*! 
\cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axSetLaserEmission(uint32_t emission_state, uint32_t which_laser);

/**
 \brief Gets swept laser emission status.
 \param[out] emitting =0 if both laser emission and sweep are disabled, =1 if both laser emission and sweep are enabled, =2 if laser emission is disabled but sweep is enabled.
 \param[in] which_laser The numeric index of the desired Laser.
 \return NO_AxERROR on success or other AxErr error code on failure.
 */	
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
	 AxErr __cdecl axGetLaserEmission(uint32_t* emitting, uint32_t which_laser);

 /**
 \brief Enables or disables pointer/aiming laser emission.
 \param[in] emission_state =1 enables pointer/aiming laser emission, =0 disables pointer/aiming laser emission.
 \param[in] which_laser The numeric index of the desired Laser.
 \return NO_AxERROR on success or other AxErr error code on failure.
 */
 /*! \cond */ AXSUN_EXPORTS /*! \endcond */
	 AxErr __cdecl axSetPointerEmission(uint32_t emission_state, uint32_t which_laser);

 /**
 \brief Gets pointer/aiming laser emission status.
 \param[out] emitting Will be non-zero if pointer/aiming laser is emitting or =0 if pointer/aiming laser is not emitting.
 \param[in] which_laser The numeric index of the desired Laser.
 \return NO_AxERROR on success or other AxErr error code on failure.
 */	
 /*! \cond */ AXSUN_EXPORTS /*! \endcond */
	 AxErr __cdecl axGetPointerEmission(uint32_t* emitting, uint32_t which_laser);

 /**
 \brief Sets swept laser emission power-up default behavior.
 \param[in] behavior The desired power-up behavior
 \param[in] which_laser The numeric index of the desired Laser.
 \return NO_AxERROR on success or other AxErr error code on failure.
 \details Saves setting to non-volatile RAM. Restart engine to observe selected power-upbehavior.
 */	
 /*! \cond */ AXSUN_EXPORTS /*! \endcond */
	 AxErr __cdecl axSetLaserPowerUp(AxPowerUpBehavior behavior, uint32_t which_laser);

 /**
\brief Gets swept laser emission power-up default behavior.
\param[out] behavior The currently configured power-up behavior
\param[in] which_laser The numeric index of the desired Laser.
\return NO_AxERROR on success or other AxErr error code on failure.
*/	
 /*! \cond */ AXSUN_EXPORTS /*! \endcond */
	AxErr __cdecl axGetLaserPowerUp(AxPowerUpBehavior* behavior, uint32_t which_laser);

/**
\brief Sets pointer/aiming laser emission power-up default behavior.
\param[in] behavior The desired power-up behavior
\param[in] which_laser The numeric index of the desired Laser.
\return NO_AxERROR on success or other AxErr error code on failure.
\details Saves setting to non-volatile RAM. Restart engine to observe selected power-up behavior. 
*/	
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
	AxErr __cdecl axSetPointerPowerUp(AxPowerUpBehavior behavior, uint32_t which_laser);

/**
\brief Gets pointer/aiming laser emission power-up default behavior.
\param[out] behavior The currently configured power-up behavior
\param[in] which_laser The numeric index of the desired Laser.
\return NO_AxERROR on success or other AxErr error code on failure.
*/	
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
	 AxErr __cdecl axGetPointerPowerUp(AxPowerUpBehavior* behavior, uint32_t which_laser);

 /**
  \brief Sets the electronic K-clock delay.
  \param[in] delay_code A 6-bit code representing the desired delay value.
  \param[in] which_laser The numeric index of the desired Laser.
  \return NO_AxERROR on success or other AxErr error code on failure.
  \details Discussion of the delay_code value can be found at https://docs.axsun.com (OCT Host page, Clock Delay subsection)
  */
 /*! \cond */ AXSUN_EXPORTS /*! \endcond */
	 AxErr __cdecl axSetClockDelay(uint32_t delay_code, uint32_t which_laser);

 /*! \brief Gets the currently configured electronic K-clock delay.
  \param[out] delay_code Will contain a 6-bit code representing the configured delay value.
  \param[in] which_laser The numeric index of the desired Laser.
  \return NO_AxERROR on success or other AxErr error code on failure.
  \details See details for axSetClockDelay().
 */
 /*! \cond */ AXSUN_EXPORTS /*! \endcond */
	 AxErr __cdecl axGetClockDelay(uint32_t* delay_code, uint32_t which_laser);

 /**
\brief Selects a pre-programmed laser drive configuration.
\param[in] which_config The desired configuration number (0, 1, 2, or 3).
\param[in] which_laser The numeric index of the desired Laser.
 \return NO_AxERROR on success or other AxErr error code on failure.
\details Laser Engines must be programmed during manufacture to support multiple drive configurations.  Contact Axsun technical support for more information.
*/
 /*! \cond */ AXSUN_EXPORTS /*! \endcond */
	 AxErr __cdecl axSetDriveConfiguration(uint32_t which_config, uint32_t which_laser);

 /**
  \brief Gets the currently selected drive configuration.
  \param[out] current_configuration The currently selected drive configuration.
  \param[in] which_laser The numeric index of the desired Laser.
  \return NO_AxERROR on success or other AxErr error code on failure.
  \details Laser Engines must be programmed during manufacture to support multiple drive configurations.  Contact Axsun technical support for more information.
  */
 /*! \cond */ AXSUN_EXPORTS /*! \endcond */
	 AxErr __cdecl axGetDriveConfiguration(uint32_t* current_configuration, uint32_t which_laser);

 /**
  \brief Gets elapsed timer values from a laser engine.
  \param[out] cum_emission_sec Number of seconds elapsed with laser in emission-on state (cumulative lifetime).
  \param[out] cum_system_sec Number of seconds elapsed with engine in powered-on state (cumulative lifetime).
  \param[out] uptime_msec Number of milliseconds elapsed in current session (since last power-up).
  \param[in] which_laser The numeric index of the desired Laser.
  \return NO_AxERROR on success or other AxErr error code on failure.
  */
 /*! \cond */ AXSUN_EXPORTS /*! \endcond */
	 AxErr __cdecl axGetElapsedTimers(uint32_t* cum_emission_sec, uint32_t* cum_system_sec, uint32_t* uptime_msec, uint32_t which_laser);

 /**
  \brief Gets the current state of a Laser Thermo-Electric Cooler (TEC).
  \param[out] TEC_state Current state of the queried laser TEC as described by the AxsunOCTControl_LW::TECState enum.
  \param[in] which_TEC The index of which TEC to query (1 or 2).
  \param[in] which_laser The numeric index of the desired Laser.
  \return NO_AxERROR on success or other AxErr error code on failure.
  \details Some Laser Engines support more than one TEC. To query the state of the primary "TEC 1", use `which_TEC` = 1, or for a secondary TEC use `which_TEC` = 2.
  */
 /*! \cond */ AXSUN_EXPORTS /*! \endcond */
	 AxErr __cdecl axGetTECState(AxTECState* TEC_state, uint32_t which_TEC, uint32_t which_laser);

 /**
 \brief Gets the current error state of a Laser device.
 \param[out] error_bitfield Current error state of the laser.
 \param[in] which_laser The numeric index of the desired Laser.
 \return NO_AxERROR on success or other AxErr error code on failure.
 \details When error_bitfield is equal to zero, there are no errors reported by the device. To decode a non-zero value of error_bitfield, contact Axsun technical support.  Use axGetTECState() to debug TEC-related errors potentially caused by insufficient thermal management.
 */
 /*! \cond */ AXSUN_EXPORTS /*! \endcond */
	 AxErr __cdecl axGetLaserErrorState(uint32_t* error_bitfield, uint32_t which_laser);

 /**
  \brief Starts the VDL home operation.
  \param[in] which_laser The numeric index of the desired Laser.
  \return NO_AxERROR on success or other AxErr error code on failure.
  \details This function returns after _requesting_ the homing operation from the VDL controller; it does not wait for the mechanical homing operation to complete before returning.
  */
 /*! \cond */ AXSUN_EXPORTS /*! \endcond */
	 AxErr __cdecl axHomeVDL(uint32_t which_laser);

 /**
  \brief Stops any VDL operation (e.g. move or home) currently in progress.
  \param[in] which_laser The numeric index of the desired Laser.
  \return NO_AxERROR on success or other AxErr error code on failure.
  */
 /*! \cond */ AXSUN_EXPORTS /*! \endcond */
	 AxErr __cdecl axStopVDL(uint32_t which_laser);

 /**
  \brief Move the VDL relative to its current position.
  \param[in] rel_position The distance to move from the current position in mm.
  \param[in] speed The speed at which to move in mm/sec.
  \param[in] which_laser The numeric index of the desired Laser.
  \return NO_AxERROR on success or other AxErr error code on failure.
  */
 /*! \cond */ AXSUN_EXPORTS /*! \endcond */
	 AxErr __cdecl axMoveRelVDL(float rel_position, float speed, uint32_t which_laser);

 /**
  \brief Move the VDL to an absolute position.
  \param[in] abs_position The new desired position in mm.
  \param[in] speed The speed at which to move in mm/sec.
  \param[in] which_laser The numeric index of the desired Laser.
  \return NO_AxERROR on success or other AxErr error code on failure.
  */
 /*! \cond */ AXSUN_EXPORTS /*! \endcond */
	 AxErr __cdecl axMoveAbsVDL(float abs_position, float speed, uint32_t which_laser);

 /**
  \brief Gets the current status of the VDL.
  \param[out] current_pos Current VDL position (mm).
  \param[out] target_pos Target VDL position (mm).  Will be different than the current position during a move or home operation.
  \param[out] speed Current speed of VDL (mm/sec).
  \param[out] error_from_last_home Accumulated error since last home operation (number of microsteps).
  \param[out] last_move_time Time for last move operation (sec).
  \param[out] state Integer codes to represent VDL state.
										 *	- 1 = idle
										 *	- 2 = moving
										 *	- 3 = seeking home
										 *	- 4 = backing off home
										 *	- 5 = at home
										 *	- 10 = error state
  \param[out] home_switch Non-zero if home (minimum position) limit switch is activated, =0 otherwise.
  \param[out] limit_switch Non-zero if maximum position limit switch is activated, =0 otherwise.
  \param[out] VDL_error Debugging information for VDL errors.
  \param[in] which_laser The numeric index of the desired Laser.
  \return NO_AxERROR on success or other AxErr error code on failure.
  */
 /*! \cond */ AXSUN_EXPORTS /*! \endcond */
	 AxErr __cdecl axGetVDLStatus(float* current_pos, float* target_pos, float* speed,
		 int32_t* error_from_last_home, uint32_t* last_move_time,
		 uint8_t* state, uint8_t* home_switch, uint8_t* limit_switch, uint8_t* VDL_error, uint32_t which_laser);

 


///@}


/** @name DAQ Device Functions
 *  These functions access DAQ devices only.
 */
 ///@{

/**
\brief Control the DAQ operational mode (Live Imaging, Burst Recording, or Imaging Off) when using the Ethernet interface.
\param[in] number_of_images =0 for Imaging Off (idle), =-1 for Live Imaging (no record), or any positive value between 1 and 32767 to request the desired number of images in a Burst Record operation.
\param[in] which_DAQ The numeric index of the desired DAQ.
\return NO_AxERROR on success or other AxErr error code on failure.
\details This function does NOT control the laser and therefore the laser emission must be enabled/disabled separately using axSetLaserEmission(). To control the DAQ operational mode when using the PCIe interface, call axImagingCntrlPCIe() in the AxsunOCTCapture library.
*/
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axImagingCntrlEthernet(int16_t number_of_images, uint32_t which_DAQ);

/**
 \brief Select the Image_sync source.
 \param[in] source The desired Image_sync source. Must be either INTERNAL, LVCMOS, LVDS, or INTERNAL_SYNC.
 \param[in] frequency The Image_sync frequency; this parameter is optional and is ignored when source is not INTERNAL or INTERNAL_SYNC.  If the source is INTERNAL, this value is interpreted in Hz and the generated Image_sync is asynchronous with the sweep trigger.  If the source is INTERNAL_SYNC, this value is interpreted in sweep triggers per image and the generated Image_sync is synchronous with the sweep trigger.
 \param[in] which_DAQ The numeric index of the desired DAQ.
 \return NO_AxERROR on success or other AxErr error code on failure.
 \details Use INTERNAL or INTERNAL_SYNC Image_sync source only for simulation and troubleshooting.  INTERNAL_SYNC requires FPGA version 6.71.0.42 or later.
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axSetImageSyncSource(AxEdgeSource source, float frequency, uint32_t which_DAQ);

/**
 \brief Select the ADC sample clock source.
 \param[in] source The desired sample clock source. Must be either EXTERNAL (Laser k-clock) or INTERNAL (500 MS/s).
 \param[in] which_DAQ The numeric index of the desired DAQ.
 \return NO_AxERROR on success or other AxErr error code on failure.
 \details Use INTERNAL sample clock source only for simulation and troubleshooting.
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axSetSampleClockSource(AxEdgeSource source, uint32_t which_DAQ);

/**
 \brief Select the sweep trigger source.
 \param[in] source The desired sweep trigger source. Must be either EXTERNAL (from Laser) or INTERNAL (100 kHz).
 \param[in] which_DAQ The numeric index of the desired DAQ.
 \return NO_AxERROR on success or other AxErr error code on failure.
 \details Use INTERNAL sweep trigger source only for simulation and troubleshooting.
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axSetSweepTriggerSource(AxEdgeSource source, uint32_t which_DAQ);

/**
 \brief Select the DAQ's pipeline mode (i.e. bypass mode) and polarization channel configuration.
 \param[in] pipeline_mode The desired pipeline mode.
 \param[in] polarization_mode The desired polarization channel (horizontal, vertical, diverse, or interleaved).
 \param[in] which_DAQ The numeric index of the desired DAQ.
 \return NO_AxERROR on success or other AxErr error code on failure.
 \details Insure that A-line subsampling has been configured with SetSubsamplingFactor() \b prior \b to setting a pipeline mode which could saturate the data interface bandwidth if subsampling is insufficient.
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axSetPipelineMode(AxPipelineMode pipeline_mode, AxChannelMode polarization_mode, uint32_t which_DAQ);

/**
 \brief Set the GAIN term during 16- to 8-bit dynamic range compression.
 \param[in] gain Gain must be between 0 and 15.999756.
 \param[in] which_DAQ The numeric index of the desired DAQ.
 \return NO_AxERROR on success or other AxErr error code on failure.
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axSetEightBitGain(float gain, uint32_t which_DAQ);

/**
 \brief Set the OFFSET term during 16- to 8-bit dynamic range compression.
 \param[in] offset Offset must be between -128.0 and 127.996094.
 \param[in] which_DAQ The numeric index of the desired DAQ.
 \return NO_AxERROR on success or other AxErr error code on failure.
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axSetEightBitOffset(float offset, uint32_t which_DAQ);

/**
 \brief Set the A-line subsampling factor.
 \param[in] subsampling_factor Subsampling factor must be between 1 and 166.
 \param[in] which_DAQ The numeric index of the desired DAQ.
 \return NO_AxERROR on success or other AxErr error code on failure.
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axSetSubsamplingFactor(uint8_t subsampling_factor, uint32_t which_DAQ);

/**
 \brief Gets (i.e. reads) the current value in a FPGA register.
 \param[in] regnum The FPGA register number to read.
 \param[out] regval The value read back from the FPGA register.
 \param[in] which_DAQ The numeric index of the desired DAQ.
 \return NO_AxERROR on success or other AxErr error code on failure.
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axGetFPGARegister(const uint32_t regnum, uint16_t* regval, uint32_t which_DAQ);

/**
 \brief Gets (i.e. reads) current values in a range of FPGA registers.
 \param[in] start_regnum The first FPGA register to read in the range.
 \param[in] end_regnum The last FPGA register to read in the range; must be greater than or equal to `start_regnum`.
 \param[out] regvals Pointer to a pre-allocated array to hold the 16-bit register values read back from the requested range of FPGA registers.
 \param[in] bytes_allocated The number of bytes pre-allocated by the user in the array pointed to by `regvals`.
 \param[in] which_DAQ The numeric index of the desired DAQ.
 \return NO_AxERROR on success or other AxErr error code on failure.
 \details The size of the pre-allocated memory pointed to by `regvals` must be 2 bytes per register in the included range:  `bytes_allocated` should equal `2 * (end_regnum - start_regnum + 1)`.
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axGetFPGARegisterRange(const uint32_t start_regnum, const uint32_t end_regnum, uint16_t* regvals, const uint32_t bytes_allocated, uint32_t which_DAQ);


/**
 \brief Gets (i.e. reads) the programmed power-on default FPGA register configuration script as register number-value pairs.
 \param[in] elements_allocated Indicates the size (<b>in number of U16 elements, NOT bytes</b>) of the pre-allocated `regnums` and `regvals` arrays.
 \param[out] regnums Pointer to a pre-allocated array to hold the 16-bit register numbers read back from the device.
 \param[out] regvals Pointer to a pre-allocated array to hold the 16-bit register values read back from the device.
 \param[out] elements_returned The number of register number-value pairs read back from the device.
 \param[in] which_DAQ The numeric index of the desired DAQ.
 \return NO_AxERROR on success or other AxErr error code on failure.
 \details `regnums` and `regvals` are arrays which must be pre-allocated by the user. `elements_returned` will never be higher than `elements_allocated`, however if `elements_returned == elements_allocated`, there may be more register number-value pairs stored on the device which were _not_ returned (indicating `elements_allocated` was too small to hold the full FPGA register configuration script).
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axGetFPGARegisterDefaults(uint32_t elements_allocated, uint16_t* regnums, uint16_t* regvals, uint32_t* elements_returned, uint32_t which_DAQ);


/**
 \brief Sets (i.e. writes) a power-on default FPGA register configuration script, overwriting the existing script.
 \param[in] regnums Pointer to an array containing 16-bit register numbers.
 \param[in] regvals Pointer to an array containing 16-bit register values.
 \param[in] count The number of register number-value pairs to be written to the device.
 \param[in] which_DAQ The numeric index of the desired DAQ.
 \return NO_AxERROR on success or other AxErr error code on failure.
 \details `regnums` points to an array containing `count` register numbers and `regvals` points to an equal-sized array of corresponding register default values.
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axSetFPGARegisterDefaults(const uint16_t* regnums, const uint16_t* regvals, uint32_t count, uint32_t which_DAQ);

/**
 \brief Sets (i.e. writes) a FPGA register with a single value.
 \param[in] regnum The FPGA register number in which to write.
 \param[in] regval The value to be written.
 \param[in] which_DAQ The numeric index of the desired DAQ.
 \return NO_AxERROR on success or other AxErr error code on failure.
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axSetFPGARegister(uint32_t regnum, uint16_t regval, uint32_t which_DAQ);

/**
 \brief Sets (i.e. writes) FPGA register bits based on a bitmask.
 \param[in] regnum The FPGA register number in which to write.
 \param[in] regval The value to be written, subject to the provided bitmask.
 \param[in] bitmask A mask indicating which bits to write (1) and which to leave unaffected (0).
 \param[in] which_DAQ The numeric index of the desired DAQ.
 \return NO_AxERROR on success or other AxErr error code on failure.
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axSetFPGARegisterMasked(uint32_t regnum, uint16_t regval, uint16_t bitmask, uint32_t which_DAQ);

/**
 \brief Sets or Clears a single bit in a FPGA register.
 \param[in] regnum The FPGA register number in which to set or clear a bit.
 \param[in] which_bit The bit number in the interval [0..15] to set or clear.
 \param[in] value =1 to Set the bit, =0 to Clear the bit.
 \param[in] which_DAQ The numeric index of the desired DAQ.
 \return NO_AxERROR on success or other AxErr error code on failure.
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axSetFPGARegisterSingleBit(uint32_t regnum, uint32_t which_bit, uint32_t value, uint32_t which_DAQ);

/**
\brief Sets (i.e. writes) the low or high byte of an FPGA register, leaving the other byte unchanged.
\param[in] regnum The FPGA register number in which to write.
\param[in] which_byte Set to = 0 for low byte, = 1 for high byte.
\param[in] value The value to be written (must be in range of 0 to 255).
\param[in] which_DAQ The numeric index of the desired DAQ.
\return NO_AxERROR on success or other AxErr error code on failure.
*/
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axSetFPGARegisterSingleByte(uint32_t regnum, uint32_t which_byte, uint32_t value, uint32_t which_DAQ);

/**
\brief Sets (i.e. writes) one nibble (4 bits) of an FPGA register, leaving the other 12 bits unchanged.
\param[in] regnum The FPGA register number in which to write.
\param[in] which_nibble Set to = 0 for lowest nibble, ... , = 3 for highest nibble.
\param[in] value The value to be written (must be in range of 0 to 15).
\param[in] which_DAQ The numeric index of the desired DAQ.
\return NO_AxERROR on success or other AxErr error code on failure.
*/
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axSetFPGARegisterSingleNibble(uint32_t regnum, uint32_t which_nibble, uint32_t value, uint32_t which_DAQ);

/**
 \brief Sets (i.e. writes) a FPGA register with an array of multiple values.
 \param[in] regnum The FPGA register number in which to write.
 \param[in] data_array Pointer to an array containing 16-bit values to be written.
 \param[in] number_of_words The number of 16-bit register values pointed to by `data_array`.
 \param[in] which_DAQ The numeric index of the desired DAQ.
 \return NO_AxERROR on success or other AxErr error code on failure.
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axSetFPGADataArray(uint32_t regnum, const uint16_t* data_array, uint32_t number_of_words, uint32_t which_DAQ);

/**
 \brief Sets (i.e. writes) an apodization window function to the FPGA.
 \param[in] channel The acquisition channels to which the window function is written.
 \param[in] window_real Pointer to an array containing the real component of the window function.
 \param[in] window_imag Pointer to an array containing the imaginary component of the window function.
 \param[in] window_length The length of the window function (i.e. the number of elements in each array describing its real and imaginary components).
 \param[in] which_DAQ The numeric index of the desired DAQ.
 \return NO_AxERROR on success or other AxErr error code on failure.
 \details A selected AxChannelMode of SUM_CHANNELS or INTERLEAVE_CHANNELS will load the provided window function into both Ch1 and Ch2 (call this function twice if each channel needs to have a differnet window function).  The values in the window_real and window_imag arrays must be on the interval from negative unity to unity (-1.0, 1.0) otherwise clipping will occur and likely result in signal distortion.  If window_length is less-than the appropriate 2^n FFT size (e.g. 2048 samples), the loaded window function will be zero-padded accordingly.
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axSetFPGAWindowFunction(AxChannelMode channel, const float* window_real, const float* window_imag, uint32_t window_length, uint32_t which_DAQ);


///@}
/** @name Internal Functions
 *  These functions are for internal or advanced users only.  Please contact Excelitas-Axsun for additional support.
 */
 ///@{


/**
* \internal
 \brief Counts the number of individual device settings elements for each of the three data types (float, int, and bool).
 \param[out] float_count The number of floating point settings.
 \param[out] int_count The number of integer settings.
 \param[out] bool_count The number of boolean settings.
 \param[in] which_device The numeric index of the desired device.
 \return NO_AxERROR on success or other AxErr error code on failure.
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axCountDeviceSettings(uint32_t* float_count, uint32_t* int_count, uint32_t* bool_count, uint32_t which_device);

/**
* \internal
 \brief Gets a floating point setting's value and name string.
 \param[in] which_setting The index of a floating point setting.
 \param[out] setting_value The floating point setting value.
 \param[out] setting_string A char array pre-allocated with at least 60 bytes into which the setting name will be copied as a nul-terminated string.
 \param[in] which_device The numeric index of the desired device.
 \return NO_AxERROR on success or other AxErr error code on failure.
 \details Use axCountDeviceSettings() to determine the number of floating point settings available (`float_count`) and then call this function in a loop while indexing `which_setting` from `0` up to `float_count - 1`.
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axGetFloatSetting(uint32_t which_setting, float* setting_value, char* setting_string, uint32_t which_device);

/**
* \internal
 \brief Gets a integer setting's value and name string.
 \param[in] which_setting The index of an integer setting.
 \param[out] setting_value The integer setting value.
 \param[out] setting_string A char array pre-allocated with at least 60 bytes into which the setting name will be copied as a nul-terminated string.
 \param[in] which_device The numeric index of the desired device.
 \return NO_AxERROR on success or other AxErr error code on failure.
 \details Use axCountDeviceSettings() to determine the number of integer settings available (`int_count`) and then call this function in a loop while indexing `which_setting` from `0` up to `int_count - 1`.
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axGetIntSetting(uint32_t which_setting, int32_t* setting_value, char* setting_string, uint32_t which_device);

/**
* \internal
 \brief Gets a boolean setting's value and name string.
 \param[in] which_setting The index of a boolean setting.
 \param[out] setting_value The boolean setting value (non-zero if TRUE, zero if FALSE).
 \param[out] setting_string A char array pre-allocated with at least 60 bytes into which the setting name will be copied as a nul-terminated string.
 \param[in] which_device The numeric index of the desired device.
 \return NO_AxERROR on success or other AxErr error code on failure.
 \details Use axCountDeviceSettings() to determine the number of boolean settings available (`bool_count`) and then call this function in a loop while indexing `which_setting` from `0` up to `bool_count - 1`.
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axGetBoolSetting(uint32_t which_setting, uint8_t* setting_value, char* setting_string, uint32_t which_device);

/**
* \internal
 \brief Write device settings to non-volatile memory (i.e. flash).
 \param[in] float_count The number of floating point settings in the `float_values` array.
 \param[in] int_count The number of integer settings in the `int_values` array.
 \param[in] bool_count The number of boolean settings in the `bool_values` array.
 \param[in] float_values An array of floating point setting values.
 \param[in] int_values An array of integer setting values.
 \param[in] bool_values An array of boolean setting values (1 = TRUE, 0 = FALSE).
 \param[in] which_device The numeric index of the desired device.
 \return NO_AxERROR on success or other AxErr error code on failure.
 \details After reading all device setting values and names (using axCountDeviceSettings(), axGetFloatSetting(), axGetIntSetting(), and axGetBoolSetting()), original (i.e. unchanged) and new (i.e. changed or updated) settings are written back to the device's non-volatile memory with this function.  Do \b NOT change the ordering or indexing of the device settings. The original values of unchanged settings \b must be written back to the device with the changed ones.
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axSettingsToDevice(uint32_t float_count, uint32_t int_count, uint32_t bool_count, const float* float_values, const int32_t* int_values, const uint8_t* bool_values, uint32_t which_device);

/**
* \internal
 \brief Disable or re-enable the Connection Heartbeat for a device connected by Ethernet network or RS-232.
 \param[in] heartbeat_state =1 to enable the Heartbeat, =0 to disable the Heartbeat.
 \param[in] which_device The numeric index of the desired device.
 \return NO_AxERROR on success or other AxErr error code on failure.
 \details The Connection Heartbeat is enabled by default when a device is connected. This function can be used to disable or subsequently re-enable it. The Connection Heartbeat sends a periodic, non-functional message to a connected device for determining its current connectivity status. If a Heartbeat response is not received, a device disconnection has occurred. If the connection Heartbeat is disabled, a disconnected device will not be recognized as such until the response to a subsequent functional message is not received.
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axConnectionHeartbeat(uint32_t heartbeat_state, uint32_t which_device);

/**
* \internal
 \brief Send a debug command.
 \param[in] command_number
 \param[in] value_out
 \param[out] command_in
 \param[out] value_in
 \param[in] which_device The numeric index of the desired device.
 \return NO_AxERROR on success or other AxErr error code on failure.
 \details This function is for Axsun internal use only.
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axDebugCommand(uint32_t command_number, uint32_t value_out, uint32_t* command_in, uint32_t* value_in, uint32_t which_device);

/**
* \internal
\brief Get debug information.
\param[in] code a debug code
\param[in,out] field1 generic data field
\param[in,out] field2 generic data field
\param[in,out] field3 generic data field
\param[in,out] field4 generic data field
\param[in,out] field5 generic data field
\param[in,out] field6 generic data field
\param[in,out] field7 generic data field
\param[in,out] field8 generic data field
\param[in,out] field9 generic data field
\param[in,out] field10 generic data field
\param[in,out] array1 generic data array
\param[in,out] array2 generic data array
\param[in] which_device The numeric index of the desired device.
\return NO_AxERROR on success or other AxErr error code on failure.
\details This function is for Axsun internal use only.
*/
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axGetDebugInfo(int32_t code, uint32_t* field1, uint32_t* field2, uint32_t* field3, uint32_t* field4, uint32_t* field5, uint32_t* field6, uint32_t* field7, uint32_t* field8, uint32_t* field9, uint32_t* field10, void* data1, void* data2, uint32_t which_device);


/**
*   \internal
 \brief Sets the DAC table.
 \param[in] points
 \param[in] speed
 \param[in] timer1time
 \param[in] timer2time
 \param[in] timer1duration
 \param[in] timer2duration
 \param[in] volts_currents
 \param[in] save_to_flash
 \param[in] which_laser The numeric index of the desired Laser.
 \return NO_AxERROR on success or other AxErr error code on failure.
 \details This function is for Axsun internal use only.
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axSetDACTable(uint32_t points, uint32_t speed, uint32_t timer1time, uint32_t timer2time,
	uint32_t timer1duration, uint32_t timer2duration, const uint16_t* volts_currents, uint32_t save_to_flash, uint32_t which_laser);

/**
*   \internal
 \brief Gets the currently configured DAC table.
 \param[in,out] points
 \param[out] speed
 \param[out] timer1time
 \param[out] timer2time
 \param[out] timer1duration
 \param[out] timer2duration
 \param[out] volts_currents
 \param[in] which_laser The numeric index of the desired Laser.
 \return NO_AxERROR on success or other AxErr error code on failure.
 \details This function is for Axsun internal use only.
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axGetDACTable(uint32_t* points, uint32_t* speed, uint32_t* timer1time, uint32_t* timer2time,
	uint32_t* timer1duration, uint32_t* timer2duration, uint16_t* volts_currents, uint32_t which_laser);

/**
*   \internal
 \brief Reads the current value in a DAQ PHY register.
 \param[in] phynum The PHY to read.
 \param[in] regnum The PHY register number to read.
 \param[out] regval The value read back from the PHY register.
 \param[in] which_DAQ The numeric index of the desired DAQ.
 \return NO_AxERROR on success or other AxErr error code on failure.
 \details This function is for Axsun internal use only.
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axReadDAQPHYRegister(uint32_t phynum, uint32_t regnum, uint16_t* regval, uint32_t which_DAQ);


/**
*   \internal
 \brief Uploads microprocessor firmware to a device from a file.
 \param[in] file Nul-terminated string containing the location and filename (i.e. absolute path) of the firmware program .bin file.
 \param[in] which_device The numeric index of the desired device.
 \return NO_AxERROR on success or other AxErr error code on failure.
 \details This function is for Axsun internal use only. This function does NOT guard against uploading data which is not a valid microprocessor program!
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axUpdateMicroprocessorFW(const char* file, uint32_t which_device);

/*!
*  \internal
 \brief Function type for a user-provided callback to be invoked during a lengthy or multi-step operation, such as during FPGA bitstream upload. This facilitates monitoring the progress while waiting for program flow to return in the main thread. */
typedef void(__cdecl* AxProgressCallbackFcn)(int32_t, int32_t, void*);

/**
*   \internal
 \brief Uploads FPGA firmware (bitstream) to a device from a file.
 \param[in] file Nul-terminated string containing the location and filename (i.e. absolute path) of the firmware program .bin file.
 \param[in] callback Function invoked periodically during upload operation (or NULL if unused).
 \param[in] user_data An arbitrary structure to be made available inside the callback function (or NULL if unused).
 \param[in] which_device The numeric index of the desired device.
 \return NO_AxERROR on success or other AxErr error code on failure.
 \details This function is for Axsun internal use only. This function attempts to guard against uploading data which is not a valid FPGA program.
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axUpdateFPGABitstream(const char* file, AxProgressCallbackFcn callback, void* user_data, uint32_t which_device);

/**
*   \internal
 \brief Uploads FPGA firmware (bitstream) to a device from a file.
 \param[in] file Nul-terminated string containing the location and filename (i.e. absolute path) of the firmware program .bin file.
 \param[in] refnumptr Reference to a LabVIEW user event to be invoked periodically during the upload operation.
 \param[in] which_device The numeric index of the desired device.
 \return NO_AxERROR on success or other AxErr error code on failure.
 \details This function is for Axsun internal use only. This function attempts to guard against uploading data which is not a valid FPGA program.
 */
/*! \cond */ AXSUN_EXPORTS /*! \endcond */
AxErr __cdecl axUpdateFPGABitstreamLabVIEW(const char* file, uint32_t* refnumptr, uint32_t which_device);

///@}


#ifdef __cplusplus
}		// end extern "C"

/*! \brief Convert AxConnectionType enum into string. */
inline const char* AxConnectTypeString(AxConnectionType connection) {
	switch (connection) {
	case AxConnectionType::USB: return "USB";
	case AxConnectionType::RS232: return "RS-232";
	case AxConnectionType::RS232_PASSTHROUGH: return "via DAQ";
	case AxConnectionType::ETHERNET: return "Ethernet";
	default: return "Unknown";
	}
}

/*! \brief Convert AxDevType enum into string. */
inline const char* AxDevTypeString(AxDevType device) {
	switch (device) {
	case AxDevType::LASER: return "Laser";
	case AxDevType::AZMYTH_LASER: return "Azmyth Laser";
	case AxDevType::EDAQ: return "EDAQ";
	case AxDevType::CLDAQ: return "CLDAQ";
	default: return "Unknown";
	}
}



#endif // __cplusplus

#endif		/* End header include guard */

